import QtQuick 2.4
import "Globals.js" as Globals

Item {
    id: root

    // default time as seconds since unix epoch
    property int currentTime: Math.round(new Date().getTime() / 1000)
    property int time: computeTimestamp()
    property bool listenKeys: false
    property int curIndex: 0

    signal exitedList()

    width: 1050
    height: 230

    function leftPressed() {
        if (curIndex > 0)
            curIndex -= 1
        else
            root.exitedList()
    }

    function rightPressed() {
        if (curIndex === keyModel.count - 1)
            root.exitedList()
        else
            curIndex += 1
    }

    Connections {
        target: main.isTft && listenKeys ? keymapper : null

        onLeftButtonPressed: {
            main.debugKeys("leftButtonPressed", "DateTimePicker")
            root.leftPressed()
        }
        onRightButtonPressed: {
            main.debugKeys("rightButtonPressed", "DateTimePicker")
            root.rightPressed()
        }
    }

    function getField(name) {
        switch (name) {
            case "MONTH":
                return monthPicker
            case "DAY":
                return dayPicker
            case "YEAR":
                return yearPicker
            case "HOURS":
                return hoursPicker
            case "MINUTES":
                return minutesPicker
            case "AM_PM":
                return amPmButton
        }
    }

    ListModel {
        id: keyModel
        ListElement { element: ""; name: "MONTH" }
        ListElement { element: ""; name: "DAY" }
        ListElement { element: ""; name: "YEAR" }
        ListElement { element: ""; name: "HOURS" }
        ListElement { element: ""; name: "MINUTES" }
        ListElement { element: ""; name: "AM_PM" }
    }

    function getMonthName(index) {
        return ["Jan", "Feb", "Mar",
                "Apr", "May", "Jun",
                "Jul", "Aug", "Sep",
                "Oct", "Nov", "Dec"][index]
    }

    // return format is "09 Jul 2010 14:22:00" (RFC 2822)
    function getDateString() {
        var year = yearPicker.value
        var month = monthPicker.value
        var day = dayPicker.value
        var hours = (hoursPicker.value % 12) + (amPmButton.value ? 12 : 0)
        var minutes = minutesPicker.value
        var timestring = Globals.padZeros(day, 2) + " "
                + getMonthName(month - 1) + " "
                + Globals.padZeros(year, 4) + " "
                + Globals.padZeros(hours,2) + ":"
                + Globals.padZeros(minutes,2)
                + ":00"
        return timestring
    }

    function computeTimestamp() {
        var timestring = getDateString()
        var ms = Date.parse(timestring)
        return Math.round(ms / 1000)
    }

    Component.onCompleted: {
        var date = new Date(currentTime * 1000)
        monthPicker.value = date.getMonth() + 1
        dayPicker.value = date.getDate()
        yearPicker.value = date.getFullYear()
        var hours = date.getHours()
        if (hours >= 12) {
            hoursPicker.value = hours === 12 ? 12 : hours % 12
            amPmButton.value = true
        } else {
            hoursPicker.value = hours === 0 ? 12 : hours
            amPmButton.value = false
        }
        minutesPicker.value = date.getMinutes()
    }

    Image {
        source: "graphics/DateTimePicker/NAV_selector_2.png"
        visible: main.isTft && listenKeys
        y: monthPicker.y + 40
        x: {
            if (root.curIndex < 0)
                return 0
            var fieldName = keyModel.get(root.curIndex).name
            if (!fieldName)
                return 0
            return getField(fieldName).x - 75
        }
    }

    Row {

        anchors { left: parent.left; top: parent.top }
        spacing: 25

        NumberPicker {
            id: monthPicker
            listenKeys: root.curIndex === 0 && root.listenKeys
            width: 100
            minValue: 1
            maxValue: 12
            valueWraps: true
            onValueChanged: {
                var daysPerMonth = [31,28,31,30,31,30,31,31,30,31,30,31]
                if (value >= 1 && value <= 12) {
                    dayPicker.maxValue = daysPerMonth[value - 1]
                }
            }
        }

        Text {
            text: "/"
            anchors.verticalCenter: parent.verticalCenter
            font.pixelSize: 72
        }


        NumberPicker {
            id: dayPicker
            listenKeys: root.curIndex === 1 && root.listenKeys
            width: 100
            minValue: 1
            maxValue: 31
            valueWraps: true
        }

        Text {
            text: "/"
            anchors.verticalCenter: parent.verticalCenter
            font.pixelSize: 72
        }


        NumberPicker {
            id: yearPicker
            listenKeys: root.curIndex === 2 && root.listenKeys
            width: 100
            minValue: 1
            maxValue: 3000
            defaultValue: 2013
            valueWraps: true
        }

        Item {
            // blank space
            width: 75
            height: 25
        }

        NumberPicker {
            id: hoursPicker
            listenKeys: root.curIndex === 3 && root.listenKeys
            width: 100
            minValue: 1
            maxValue: 12
            valueWraps: true
        }

        Text {
            text: ":"
            anchors.verticalCenter: parent.verticalCenter
            font.pixelSize: 72
        }


        NumberPicker {
            id: minutesPicker
            listenKeys: root.curIndex === 4 && root.listenKeys
            width: 100
            minValue: 0
            maxValue: 59
            valueWraps: true
            zeroPadding: 2
        }

        Text {
            id: amLabel
            width: 30
            height: 25
            anchors.verticalCenter: parent.verticalCenter
            text: "AM"
        }

        ToggleButton {
            id: amPmButton
            width: 60

            Connections {
                target: main.isTft && root.curIndex === 5 && root.listenKeys ? keymapper : null

                onUpButtonPressed: {
                    main.debugKeys("upButtonPressed", "DateTimePicker", "(am/pm button)")
                    amPmButton.value = !amPmButton.value
                }

                onDownButtonPressed: {
                    main.debugKeys("downButtonPressed", "DateTimePicker", "(am/pm button)")
                    amPmButton.value = !amPmButton.value
                }
            }
            anchors.verticalCenter: parent.verticalCenter
        }

        Text {
            id: pmLabel
            width: 30
            height: 25
            anchors.verticalCenter: parent.verticalCenter
            text: "PM"
        }
    }
}
