import QtQuick 2.4

// This element manages audio priority and tracks requests from audio sources to play audio to user.

// The AudioManager accepts requests to play audio via several xxxRequest functions, where xxx is ipod, tv, va, etc.
// It responds by modifying a set of xxxAllowed properties, which indicate that it is OK for the corresponding source to play audio.
// The ipod source is coded as the highest priority source, and if playing, all other sources are denied.

Item {
    id: audioManager

    property int volume: 0
    property bool mute: false
    property bool btconnected: false
    property bool ipodPlaying: false
    property bool ipodPlayingVideo: false
    property bool tvPlaying: false
    property bool virtActivePlaying: false
    property bool netpulsePlaying: false
    property bool flashPlaying: false

    property bool ipodAllowed: false
    property bool tvAllowed: false
    property bool virtActiveAllowed: false
    property bool netpulseAllowed: false
    property bool ipodVideoAllowed: false
    property bool tvVideoAllowed: false
    property bool virtActiveVideoAllowed: false
    property bool netpulseVideoAllowed: false
    property bool flashAllowed: false
    property bool btAllowed: false
    property string currentMuxSource: ""

    property variant audioSources: []
    property string videoSource: ""

    property bool audioPlaying: audioSources.length > 0
    property bool debug: false

    function debugPrint(message) {
        if(debug)
            errorManager.debugMessage("AudioManager: " + message)
    }

    function reset() {
        debugPrint("reset")

        ipodAllowed = false
        virtActiveAllowed = false
        netpulseAllowed = false
        tvAllowed = false

        virtActiveVideoAllowed = false
        netpulseVideoAllowed = false
        ipodVideoAllowed = false
        tvVideoAllowed = false
        flashAllowed = false

        btAllowed = false

        setMux("none")
        audioManager.volume = 24

        rebuildSourceList()
    }

    function setVolume(percent) {
        audioManager.volume = percent
    }

    function setMute(muted) {
        audioManager.mute = muted
    }

    onMuteChanged: {
    }

    onVolumeChanged: {
        audioManager.mute = false
    }

    function setMux(source) {
        debugPrint("set audio mux: " + source)
        switch(source) {
            case "none":{
                if (currentMuxSource !== "none") {
                    brainboard.setNoneAVSource()
                    currentMuxSource = "none"
                }
                break;
            }
            case "ipod":{
                if (currentMuxSource !== "ipod") {
                    brainboard.setiPodAVSource()
                    currentMuxSource = "ipod"
                }
                break;
            }
        }
    }

    function resyncMuxSource() {
        if (ipodAllowed || ipodVideoAllowed)
            setMux("ipod")
        else
            setMux("none")
    }

    function getCurrentSource() {
            if (audioManager.ipodAllowed || audioManager.ipodVideoAllowed)
                return "ipod"
            else if (audioManager.btAllowed)
                return "bt"
            else if (audioManager.tvAllowed)
                return "tv"
            else if (audioManager.virtActiveAllowed)
                return "va"
            else if (audioManager.flashAllowed)
                return "flash"
            else
                return ""
    }

    function nextSource() {
        var currentSource = getCurrentSource()
        // find source in list
        var currentSourceIndex = -1
        for (var i = 0; i < audioSources.length; i++) {
            if (audioSources[i] === currentSource) {
                currentSourceIndex = i
                break
            }
        }

        var nextSrc = audioSources[(currentSourceIndex + 1) % audioSources.length]

        if (nextSrc)
            enableSource(nextSrc)
    }

    function ipodRequest() {
        debugPrint("ipod request")
        if(!ipodAllowed)
            enableSource("ipod")
    }

    function ipodVideoRequest() {
        debugPrint("ipod video request")
        if(!ipodVideoAllowed) {
            enableSource("ipod")
            tvVideoAllowed = false
            virtActiveVideoAllowed = false
            netpulseVideoAllowed = false
            ipodVideoAllowed = true
            flashAllowed = false
            videoSource = "ipod"
        }
    }

    function tvRequest() {
        debugPrint("tv request")
        _lastRequested = "tv"
        if(!tvAllowed)
            enableSource("tv")
    }

    function vaRequest() {
        debugPrint("va request")
        _lastRequested = "va"
        if(!virtActiveAllowed)
            enableSource("va")
    }

    function netpulseRequest() {
        if(!netpulseAllowed)
            enableSource("netpulse")
    }

    function flashRequest() {
        _lastRequested = "flash"
        if(!flashAllowed)
            enableSource("flash")
    }

    function btRequest() {
        if(!btAllowed)
            enableSource("bt")
    }

    Component.onCompleted: reset()

    property string _lastRequested   // string name of last source which will be re-enabled when ipod stops
    property string _lastEnabled    // used to revert back to previous source when netpulse is navigated away from

    onTvPlayingChanged: rebuildSourceList()
    onVirtActivePlayingChanged: rebuildSourceList()
    onNetpulsePlayingChanged: rebuildSourceList()
    onIpodPlayingChanged: rebuildSourceList()
    onBtconnectedChanged: rebuildSourceList()
    onFlashPlayingChanged: rebuildSourceList()

    onIpodAllowedChanged: resyncMuxSource()
    onIpodVideoAllowedChanged: resyncMuxSource()

    onVideoSourceChanged: debugPrint("Video Source:" + videoSource)

    function disableNetpulse() {
        console.log("disableNetpulse() called, re-enabling", _lastEnabled)
        if (_lastEnabled !== "") {
            enableSource(_lastEnabled)
        }
    }

    function enableSource(name) {
        debugPrint("enable audio source " + name)

        if (name !== "netpulse") {
            _lastEnabled = name
        }

        switch(name) {
            case "ipod": {
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                ipodAllowed = true
                flashAllowed = false
                btAllowed = false

                break;
            }
            case "tv": {
                ipodAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                tvAllowed = true
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "tv"

                setMux("none")

                break;
            }
            case "va": {
                ipodAllowed = false
                tvAllowed = false
                netpulseAllowed = false
                virtActiveAllowed = true
                netpulseVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "va"

                setMux("none")

                break;
            }
            case "netpulse": {
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = true
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "netpulse"

                setMux("none")

                break;
            }
            case "flash": {
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                ipodVideoAllowed = false
                //tvVideoAllowed = false
                //virtActiveVideoAllowed = false
                netpulseVideoAllowed = false
                flashAllowed = true
                btAllowed = false

//                videoSource = "netpulse" // set to flash?

                setMux("none")

                break;
            }

            case "bt": {
                ipodAllowed = false
                tvAllowed = false
                netpulseAllowed = false
                virtActiveAllowed = false
                flashAllowed = false
                btAllowed = true

                setMux("none")
                break
            }
        }
    }

    // update string-based list of currently playing sources
    function rebuildSourceList() {
        var newSources = ""
        newSources = []
        if(ipodPlaying)
            newSources = newSources.concat(["ipod"])
        if(btconnected)
            newSources = newSources.concat(["bt"])
        if(tvPlaying) {
            newSources = newSources.concat(["tv"])
        } else if (videoSource === "tv") {
            videoSource = ""
        }

        if(virtActivePlaying)
            newSources = newSources.concat(["va"])
        else if(videoSource == "va") {
            videoSource = ""
        }
        if(netpulsePlaying)
            newSources = newSources.concat(["netpulse"])
        else if(videoSource == "netpulse") {
            videoSource = ""
        }
        if (flashPlaying)
            newSources = newSources.concat(["flash"])

        audioSources = newSources
    }
}
