import QtQuick 2.4
import Process 0.1
import FileIO 1.0
import Components 1.0

import ".."
import "../Keypad"
import "../Television"

Item{
    id: root
    anchors.fill: parent

    property int fontSize: 24
    property int topOffset: -150
    property int itemSpacing: 12
    property int itemColumnSpacing: 50
    property bool debug: false
    property bool iptvPopupHasBeenShown: false

    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property string netpulseVersion: ""

    onIsUsbDeviceLoadedChanged: print("USB DEVICE: " + isUsbDeviceLoaded)

    function filterCountries(tuner) {
        if (tuner === "DVB-C") {
            tuner = "DVB"
        }
        var newModel = country_codes

        if (tuner === "MGT") {
            // Show all for MGT
        } else {
            for( var i = country_codes.count-1 ; i >= 0 ; i--) {
                if ( newModel.get(i).tuner !== tuner ) {
                    newModel.remove(i)
                }
            }
        }
        countryListPopup.model = newModel
    }

    function  maybeShowWifiConfig() {
        if (!netConnections.wiredConfigured && !netConnections.wirelessConfigured) {
            wifiBrowser.visible = true
        }
    }

    function debugLog(string) {
        if(root.debug)
            print("network: " + string)
    }

    LayoutMirroring.enabled: Qt.application.layoutDirection === Qt.RightToLeft
    LayoutMirroring.childrenInherit: true

    Component.onCompleted: {
        netConnections.refreshList()
        filterCountries(tv.tunerType)
        homeUrlEdit.text = options.homeURL
        getNetpulseVersion.start()
    }

    Process{
        id: ifconfigProcess
        program: "ifconfig"
        arguments: ["p2p1"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "p2p1")
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }

        onFinished: {
            if (!root.success) {
                ifconfigProcessWireless.start()
            }
        }
    }

    Process {
        id: iwconfigProcess
        program: "iwconfig"
        arguments: ["wlan0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "iwconfig output: " + output)
            if(output.match(/wlan0/)) {
                var essidSearch = output.match(/ESSID:"(.+?)"\s/)
                root.currentESSID = essidSearch ? essidSearch[1] : qsTr("not connected","wireless network")
            }
        }
    }

    Process {
        id: configureNetworkProcess

        onStarted: { reconfiguringNotice.show() }
        onFinished: {
            netConnections.refreshList()
            netConnectionList.refresh()
            dataExport.getNetworkAddresses()
            reconfiguringNotice.hide()
            sapLoader.reload()
        }

        program: "scripts/configure_network.sh"
        arguments: ["p2p1"]
    }


    Process {
        id: getNetpulseVersion
        program: "scripts/npversion.sh"
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            root.netpulseVersion = output
        }
    }

    NetworkConnectionList {
        id: netConnectionList
        preventEthDisconnect: root.iptvPopupHasBeenShown
        anchors {
            top: parent.top
            left: parent.left
            leftMargin: 160
            topMargin: 20
        }

        onDisconnectRequest: {
            console.log("Disconnect from", interfaceName)
            if (interfaceName === "p2p1" && options.iptvEnabled) {
                cannotRemovePopup.show()
                cannotRemovePopup.startTimeout(2)
                // Now disable the X button
                root.iptvPopupHasBeenShown = true
                return
            }

            disconnectPopup._show()
            disconnectNetwork.arguments = [interfaceName]
            disconnectNetwork.start()
        }
    }

    MessagePopup {
        id: cannotRemovePopup
        message: qsTr("This connection is in use by IPTV. Please disable IPTV feature to remove this connection.")
        visible: false
    }

    MessagePopup {
        id: disconnectPopup
        message: qsTr("Disconnecting...") + application.emptyString
        property int minDisplayTime: 2000
        property bool done: false

        Timer {
            id: minDisplayTimer
            interval: disconnectPopup.minDisplayTime
            repeat: false
            running: false
            onTriggered: {
                if (disconnectPopup.done) {
                    disconnectPopup.hide()
                }
            }
        }

        function _show() {
            done = false
            minDisplayTimer.restart()
            show()
        }

        function _hide() {
            done = true
            if (minDisplayTimer.running) {
                return
            }
            disconnectPopup.hide()
        }
    }

    Process {
        id: disconnectNetwork
        program: "scripts/disconnect_network.sh"
        arguments: ["p2p1"]
        onFinished: {
            disconnectPopup._hide()
            netConnections.refreshList()
            netConnectionList.refresh()
        }
    }

    ImageButton {
        id: interfaceButton
        width: 70
        height: 50

        label_under: true
        label_color: "black"
        upimage: "../graphics/205/SM_Button_Up.png"
        downimage: "../graphics/205/SM_Button_Down.png"
        label: qsTr("setup connection")
        anchors {
            right: parent.right
            top: parent.top
            topMargin: 10
            rightMargin: netConnections.list.count > 0 ? 150 : 550
        }
        onClicked: {
            wifiBrowser.visible = true
        }
    }

    Grid {
        id: labels

        columns: 2
        spacing: itemSpacing
        anchors { left: parent.left ; leftMargin: 200 ; right: parent.right ; rightMargin: 200 }
        anchors { verticalCenter: parent.verticalCenter ;  verticalCenterOffset: topOffset }

        Row {
            spacing: 170

        }

        // Bluetooth MAC
//        Text{
//            text: qsTr("Bluetooth Address") + application.emptyString
//            font.pointSize: fontSize
//        }
//        Text{
//            text: bluetoothAdapter.address ? bluetoothAdapter.address : qsTr("Not Installed") + application.emptyString
//            font.pointSize: fontSize
//        }

        // Netpulse Version
//        Text {
//            text: qsTr("Netpulse Version") + application.emptyString
//            font.pointSize: fontSize
//        }
//        Text {
//            text: root.netpulseVersion
//            font.pointSize: fontSize
//        }
    }

    Item {
        id: browserEnableRow
        anchors { top: netConnectionList.bottom ; topMargin: 30 ; left: netConnectionList.left }
        width: 400
        height: childrenRect.height

        Text {
            text: qsTr("Web Browser Enabled") + application.emptyString
            anchors.left: parent.left
        }

        ToggleButton {
            id: browserEnabledMode
            anchors.right: parent.right

            value: options.browserEnabled

            onValueChanged:
            {
                options.browserEnabled = value ? true : false
            }
        }
    }

    MouseArea {
        id: webEnableMouseArea

        anchors.fill: browserEnableRow
        onClicked: {
            if (browserEnabledMode.value === true)
                browserEnabledMode.value = false
            else
                webTermsPrompt.show()
        }
    }

    Row {
        id: homeUrlRow

        anchors { top: browserEnableRow.bottom ; topMargin: 10 ; left: netConnectionList.left ; right: parent.right ; rightMargin: 20 }

        spacing: 5
        visible: options.browserEnabled

        Text {
            id: urlLabel

            text: qsTr("Homepage:")
            width: paintedWidth
            height: paintedHeight
        }

        TextInput {
            id: homeUrlEdit

            width: parent.width - urlLabel.width - 90

            Rectangle { color: "white" ; border.color: "black" ; anchors.fill: parent ; z: parent.z-1 }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    networkKeypad.setValue(homeUrlEdit.text)
                    networkKeypad.targetItem = homeUrlEdit
                    networkKeypad.show()
                }
            }
        }

        // get url result typed in alpha keypad
        Connections {
            target: networkKeypad.targetItem === homeUrlEdit ? networkKeypad : null
            ignoreUnknownSignals: true

            onValueEntered: {
                networkKeypad.hide()
                homeUrlEdit.text = val
                networkKeypad.clear()
                networkKeypad.targetItem = null
                options.homeURL = homeUrlEdit.text
            }
        }
    }

    Row {
        id: linksRow
        visible: options.browserEnabled
        anchors { top: homeUrlRow.bottom; topMargin: 30 ; left: netConnectionList.left ; right: parent.right ; rightMargin: 20 }
        spacing: 40

        Text {
            id: linksLabel
            text: qsTr("Region") + ":" + application.emptyString
        }

        Rectangle {
            color: "#cccccc"
            radius: 10
            width: 200
            height: 30

            Text {
                id: countryValue
                text: qsTranslate("TelevisionRegionList", countryListPopup.selectedData.label) + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                font.bold: true
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    countryListPopup.visible = true
                }
            }
        }
    }

    Item {
        visible: main.isTouch
        id: netpulseSetupControls

        anchors { top: linksRow.visible ? linksRow.bottom : browserEnableRow.bottom ; topMargin: 30 ; left: netConnectionList.left }
        width: 400

        Text {
            text: qsTr("Netpulse Enabled") + application.emptyString
            anchors.left: parent.left
        }

        ToggleButton {
            id: netpulseEnabledMode
            value: options.netpulseEnabled
            anchors.right: parent.right
            onValueChanged:
            {
                options.netpulseEnabled = value ? true : false
                if (options.netpulseEnabled) {
                    maybeShowWifiConfig()
                }
            }
        }
    }

    ImageButton {
        id: netpulseInstallButton

        anchors {
            left: netpulseSetupControls.right
            leftMargin: 70
            verticalCenter: netpulseSetupControls.verticalCenter
            verticalCenterOffset: 20
        }

        label: qsTr("Configure Netpulse") + application.emptyString
        label_under: true
        label_color: "black"
        upimage: "../graphics/205/SM_Button_Up.png"
        downimage: "../graphics/205/SM_Button_Down.png"

        onClicked: {
            netpulseInstall.show()
            debugLog("showing netpulse install page")
        }
    }

    AlphaKeypad {
        id: networkKeypad
        visible: false
        anchors { horizontalCenter: parent.horizontalCenter ; bottom: parent.bottom }
        z: 9999
        onClose: hide()
    }

    CountryListPopup {
        id: countryListPopup
        function saveResult() {
            if (countryListPopup.selectedData) {
                var code = countryListPopup.selectedData.code
                options.browserCountryCode = code
                homeUrlEdit.text = "http://shop.truefitness.com/console_links/" + code
                options.homeURL = homeUrlEdit.text
            }
        }
        onModelChanged: {
            if (!model)
                return

            // Initialize to current value
            for (var i = 0; i < model.count; i++) {
                if (model.get(i).code === options.browserCountryCode) {
                    currentIndex = i
                    return
                }
            }
            currentIndex = -1
        }

        anchors {
            left: parent.left
            top: parent.top
            topMargin: -20
        }
        model: country_codes
        modelTranslationContext: "TelevisionRegionList"
        onCurrentIndexChanged: {
        }
        onItemSelected: {
            close()
            saveResult()
        }
        onClose: {
            saveResult()
        }
    }

    TelevisionRegionList {
        id: country_codes
    }

    MessagePopup {
        id: reconfiguringNotice

        message: qsTr("Reconfiguring Network Settings...") + application.emptyString
    }

    WifiBrowser {
        id: wifiBrowser

        visible: false
        onSelected: {
            visible = false
            var args = []
            if (options.iptvEnabled) {
                args.push("-b")
            }
            args = args.concat([accessPoint.interface, accessPoint.essid, accessPoint.password])

            configureNetworkProcess.arguments = args

            if (options.iptvEnabled) {
                // IPTV allows multiple connections
                // So we don't need to prompt the user
                configureNetworkProcess.start()
            } else {
                if (netConnections.wiredConfigured || netConnections.wirelessConfigured) {
                    // They already have a connection. Check if they want to disconnect first
                    disconnectPrompt.show()
                } else {
                    // Add new connection
                    configureNetworkProcess.start()
                }
            }

            return

        }
        onCanceled: visible = false
        anchors { horizontalCenter: parent.horizontalCenter ; verticalCenter: parent.verticalCenter }
    }

    ConfirmationPrompt {
        id: disconnectPrompt
        property string network: netConnections.wiredConfigured ? "Wired Ethernet" : netConnections.currentESSID
        x: 300 ; y: 200
        visible: false
        prompt: qsTr("This will disconnect you from ") + network + ". " + qsTr("Are you sure?") + application.emptyString
        onNoClicked: {
            configureNetworkProcess.arguments = []
            hide()
        }
        onYesClicked: {
            configureNetworkProcess.start()
            hide()
        }
    }

    PopupWebAgreement {
        id: webTermsPrompt

        Component.onCompleted: hide()
        x: 150
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: 80
        anchors.verticalCenter: parent.verticalCenter

        onYesClicked: {
            hide()
            browserEnabledMode.value = true
            maybeShowWifiConfig()
        }

        onNoClicked: {
            hide()
            browserEnabledMode.value = false
        }
    }
}
