import QtQuick 2.4
import StorageMgr 1.0
import BootLoader 1.0
import ".."

Item {
    id: root

    property bool firmwareUpdating: false
    property string installedFirmwareFile
    property string firmwareUpdateFile: (storageMgr.firmwareFilename.length > 0) ? storageMgr.firmwareFilename : installedFirmwareFile
    property string firmwareUpdateFileShortname
    property string firmwareUpdateVersion: versionFromFilename(firmwareUpdateFile)
    property int progress: bootloader.progress
    property bool debug: false
    property bool fromUSB: false

    property bool previouslyinbrainboardmode: false

    signal firmwareFound(string version)
    signal faultMsg(string message)
    signal status(string message)
    signal updateStarted()
    signal updateFinished()

    function firmwareUpdateAvailable() {
        if (!root.firmwareUpdateFile)
            return false

        console.log("Comparing firmware versions. Available = ", versionFromFilename(root.firmwareUpdateFile), "; Current = ", brainboardVersion(machineinfo.brainboardFirmwareVersion))
        return (versionFromFilename(root.firmwareUpdateFile) !== brainboardVersion(machineinfo.brainboardFirmwareVersion))
    }

    function debugPrint(message) {
        if(root.debug)
            print("FirmwareLoader: " + message)
    }

    Component.onCompleted: {
        debugPrint("Initializing Firmware Loader...")
        previouslyinbrainboardmode = brainboard.enabled
        checkHexFileUpdate()
    }

    function checkForUpdates() {
        debugPrint('looking for hex files')
        storageMgr.pollHexFiles()
    }

    // Find the firmware hex file installed by debian package
    function checkHexFileUpdate() {
        var hexfiles = storageMgr.listFiles("/data/firmware/", "*.hex")
        if(hexfiles.length > 0) {
            root.installedFirmwareFile = "/data/firmware/" + hexfiles[0]
        }
    }

    function loadFirmware(filename) {
        if(brainboard.enabled){
            brainboard.goToBootloader()

            status(qsTr("Preparing firmware update...") + application.emptyString)
            debugPrint("waiting for bootloader..." + enabled)
            brainboard.isUpdating = true
        }
        updateStarted()
        root.firmwareUpdating = true
        status(qsTr("Starting Firmware Load..."))
        debugPrint("starting load...")
        bootloader.loadFirmware( filename )
    }

    function brainboardVersion(version) {
        return version.substr(0, version.lastIndexOf("."))
    }

    function versionFromFilename(filename) {
        if(!filename)
            return ""
        var parts = filename.split("/")
        if(parts.length <= 0)
            return 0;
        parts = parts[parts.length-1].split(".")
        parts = parts[0].split("_")
        var version = parts[1] + "." + parts[2]
        if(parts.length > 3)
            version += "." + parts[3]

        return version
    }

    onFirmwareUpdateFileChanged: {
        var file = root.firmwareUpdateFile
        file = file.replace("/media/usb0/", "USB: ")
        file = file.replace("/data/firmware/", "")
        file = file.replace("BBFW_", qsTr("Version "))
        file = file.replace(/_/g, ".")
        file = file.replace(".hex", "")
        root.firmwareUpdateFileShortname = file
        debugPrint("update file changed:" + file)
        root.fromUSB = storageMgr.firmwareFilename.match("usb") ? true : false
        if (firmwareUpdateAvailable()) {
            firmwareFound(firmwareUpdateVersion)
        }
        return qsTr("Firmware to load: ") + application.emptyString + file
    }

    onFirmwareUpdatingChanged: {
        debugPrint("updating changed:" + firmwareUpdating)

        if(firmwareUpdating === false && brainboard.isUpdating === true){
            brainboard.isUpdating = false
            debugPrint("update finished.")
            status(qsTr("Update Complete."))
            updateFinished()
        }
    }

    BootLoader {
        id: bootloader

        onBootloadReasonChanged: {
            if(bootloadreason !== 0){
                //faultMsg("Fault: " + bootloadreason +"\r\nFlash: "+flashchecksum +"\r\nCalculated: "+calculatedchecksum)
                debugPrint("fault bootload reason:" + bootloadreason)
            }
        }

        onMessage: {
            debugPrint("message:" + msg)
            status(msg)
        }

        onFault: {
            debugPrint("fault:" + msg)
            faultMsg(qsTr("Fault: " + msg))
            root.firmwareUpdating = false
        }

        onEnabledChanged: {
            debugPrint("bootloader enabled:" + enabled)
            if(enabled)
            {
                if(previouslyinbrainboardmode){
                }
            }
            else {
                root.firmwareUpdating = false
            }
        }
    }
}
