// Keypad.qml - Keypad control for entering numeric information.

import QtQuick 2.4
import Components 1.0

import "../"

Item {
    id: root

    signal valueEntered (string val)
    signal keyPress(string key)
    signal close()

    property variant keys_model: number_keys
    property string units: ""
    property bool allow_decimal: false
    property int decimalPlaces:0
    property int length:6
    property bool allow_close: true
    property bool show_keys: true
    property bool show_entry: true
    property bool clear_after_enter: true
    property string instructions: ""
    property variant keyEntryText: ""
    property string value: decimalPlaces ? decimalFormat(keyEntryText) : keyEntryText
    property string icon : ""

    property bool _enterPressed: false

    height: show_entry ? 475 : 325
    width: 322

    function clear() {
        _enterPressed = false
        keyEntryText = ""
    }

    function setValue(val) {
        keyEntryText = val
        _enterPressed = true  // next keypress will first clear entry
    }

    function submit() {
        add("ENTER")
    }

    onKeyPress: {
        add(key);
    }

    function add(val) {
        // clear the entry first if the old value is still shown after pressing enter
        if (keyEntryText[0] === "-" )
            keyEntryText = ""
        if (val[0] === "<" )
        {
            keyEntryText = keyEntryText.slice(0,-1)
            val = ""
        }
        else if (val === "ENTER" )
        {
            valueEntered(value)
            if (root.clear_after_enter)
                keyEntryText = ""
            root._enterPressed = true
            return
        }
        else {
            if(root._enterPressed)
                clear()
            keyEntryText = rollover(keyEntryText+val)
        }
    }

    function decimalFormat(text)
    {
        var str = text

        if(str.length==0)
            str = "0"
        var val = parseInt(str,10)
        val = val / Math.pow(10, root.decimalPlaces)
        val = val.toFixed(root.decimalPlaces)
        return val
    }

    function rollover(text,character)
    {
        var string = text
        var newstr
        if(text.length > root.length)
        {
            newstr = text.slice(1)
        }
        else newstr = text
        return newstr
    }

    // stack top entry box on top of keypad
    Image {
        id: keypad_top

        visible: show_entry
        source: "../graphics/224/KEYPAD_topshape.png"
        height: visible ? 127 : 0

        // rounded rectangle where entered data is displayed
        Rectangle {
            id: entrybox
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 5
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: -25
            // width of entry box depends on length of allowed entry, subject to minimum size
            width: Math.max(180, root.length *22 + units_text.width)//216
            height: 45
            Text {
                id: entrytext
                text: root.value
                font.pointSize: 32
                font.bold: true
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: units_text.left
                anchors.rightMargin: 5
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }

            // optionally display units
            Text {
                id: units_text
                text: root.units
                font.bold: true
                font.pointSize: 12
                anchors.right: parent.right
                anchors.rightMargin: 5
                anchors.verticalCenter: parent.verticalCenter
                // show units only when some numbers are displayed
                visible: keyEntryText && keyEntryText[0] !== '-' ? true : false
            }
        }

        ImageButton {
            id: close_button
            visible: root.allow_close
            onClicked: close()
            upimage: "../graphics/netpulse/GRAYbutton_video.png"
            downimage: "../graphics/netpulse/GRAYbutton_video_PRESSED.png"
            anchors.verticalCenter: entrybox.verticalCenter
            anchors.left: entrybox.right
            anchors.leftMargin: 0
            label: "X"
            label_color: "black"
            fontsize: 34
            label_x_offset: 1
            label_y_offset: 1
        }


        Text {
            id: instruction_text
            color: "#ffffff"
            visible: root.instructions != ""
            text: root.instructions
            anchors.horizontalCenter: parent.horizontalCenter
            font.bold: true
            font.family: "Liberation Sans"
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 16
        }

        Image{
            id:keypadIcon
            source: root.icon
            anchors.verticalCenter: entrybox.verticalCenter
            anchors.left: entrybox.left
            anchors.leftMargin: -15

        }
    }
    // main keypad body
    BorderImage {
        id: keypad_box

        visible: root.show_keys
        source: "keypad_shape.png"
        anchors.top: keypad_top.bottom
        anchors.topMargin: -37
        anchors.horizontalCenter: keypad_top.horizontalCenter
        x: 10
        y: 90
        width: 347
        height: 401
        anchors.horizontalCenterOffset: 0
        border.bottom: 70
        border.top: 45
        border.right: 45
        border.left: 45
        smooth: true

        MouseArea{
            id: keypadBodyCover
            anchors.fill: parent
        }

        Column {
            id: keypad_column
            anchors.top: parent.top
            anchors.topMargin: 45
            anchors.horizontalCenter: parent.horizontalCenter
            spacing: 5
            Grid {
                id: keygrid
                rows: 4
                columns: 3
                spacing: 12
                anchors.horizontalCenter: parent.horizontalCenter

                Repeater {
                    model: keys_model
                    delegate: keypad_button
                }
            }
        }
    }

    // delegate for keypad button
    Component {
        id: keypad_button
        ImageButton {
            upimage: model.buttonimage ? model.buttonimage : ""
            downimage: "keypad_orangebutton.png"
            icon: image ? image : ""
            label: size ? value : ""
            label_color: "black"
            label_visible: (application.currentLanguage !== "en" && value === "ENTER") ? false : true
            fontsize: size
            label_x_offset: x_off
            label_y_offset: y_off

            enabled: true
            onClicked: keyPress(value)
        }
    }

    // model of keypad keys
    ListModel {
        id: number_keys
        ListElement { value: "1" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "2" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "3" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "4" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "5" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "6" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "7" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "8" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "9" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        // this is the backspace key.  The '<' value indicates to the entry function to delete previous character
        ListElement { value: "<" ; image: "keypad_backspaceicon.png"; buttonimage: "keypad_graybutton.png"; size: 0; x_off: 0; y_off: 0}
        ListElement { value: "0" ;buttonimage: "keypad_graybutton.png"; size: 45; x_off: 0; y_off: 0}
        ListElement { value: "ENTER" ; image: "keypad_returnicon.png" ;buttonimage: "keypad_bluebutton.png"; size: 14; x_off: 0; y_off: -10}
    }
}
