import QtQuick 2.4
import Process 0.1
import StorageMgr 1.0

// Holds a list of active network connections
Item {
    id: root

    property bool isRefreshing: refreshTimer.running || ifconfigProcess.processState == Process.Running || ifconfigProcessWireless.processState == Process.Running || iwconfigProcess.processState == Process.Running || wpaCliProcess.processState == Process.Running

    // Wired connection information
    property bool wireDisconnected: false
    property bool wiredConfigured: false
    property bool wiredEnabled: false
    property string wiredIp
    property string wiredBroadcast
    property string wiredNetmask
    property string wiredMac
    // Wireless connection information
    property bool wirelessConfigured: false
    property bool wirelessEnabled: false
    property bool encrypted: false
    property string wirelessIp
    property string wirelessBroadcast
    property string wirelessNetmask
    property string wirelessMac
    property string signalStrengthRatio
    property string signalStrength: signalStrengthText(signalStrengthRatio) + application.emptyString
    property string currentESSID: qsTr("not connected","wireless network")
    property bool debug: true

    property alias list: activeConnections

    Component.onCompleted: {
        refreshList()
    }

    onWireDisconnectedChanged: {
    }

    function debugLog(str) {
        if (root.debug) {
            console.log(str)
        }
    }

    function fetchConfigured() {
        // Find out which interfaces have files in /etc/network/interfaces.d, regardless
        // of whether or not they are currently active
        var files = storageMgr.listFiles("/etc/network/interfaces.d", "*.cfg")
        for (var i = 0; i < files.length; i++) {
            if (files[i].indexOf("p2p1") >= 0) {
                root.wiredConfigured = true
            } else if (files[i].indexOf("wlan0") >= 0) {
                root.wirelessConfigured = true
            }
        }
    }

    function signalStrengthText(ratio) {
        var text = ""
        if (ratio.indexOf("/") < 0) {
            text = qsTr("Unknown")
        } else {
            var parts = ratio.split("/")
            var num = parseInt(parts[0], 10), den = parseInt(parts[1], 10)
            var numRatio = (1.0 * num) / den
            if (numRatio >= 0.8) {
                text = qsTr("Very Strong")
            } else if (numRatio >= 0.6) {
                text = qsTr("Strong")
            } else if (numRatio >= 0.4) {
                text = qsTr("Weak")
            } else {
                text = qsTr("Very Weak")
            }
        }

        return text
    }

    function parseString(str, name)
    {
        // ifconfig still returns output for interfaces that are down, so filter here to
        // make sure it is up before continuing.
        if (str.indexOf("UP") === -1) {
            return
        }

        var ipSearch = str.match(/inet addr:(.+?) /)
        if (ipSearch === null) {
            return // no IP found. Abort and do not update values
        }

        var ip = ipSearch[1]
        var broadcast = str.match(/Bcast:(.+?) /)[1]
        var mask = str.match(/Mask:(.+?)\s/)[1]
        var mac = str.match(/HWaddr (.+?)\s/)[1]
        if (name === "wlan0") {
            root.wirelessEnabled = true
            root.wirelessBroadcast = broadcast
            root.wirelessNetmask = mask
            root.wirelessIp = ip
            root.wirelessMac = mac
        } else if (name === "p2p1") {
            root.wiredEnabled = true
            root.wiredBroadcast = broadcast
            root.wiredNetmask = mask
            root.wiredIp = ip
            root.wiredMac = mac
        }
    }

    function resetFields() {
        activeConnections.clear()
        root.wiredConfigured = false
        root.wirelessConfigured = false
        root.wirelessEnabled = false
        root.encrypted = false
        root.wirelessBroadcast = ""
        root.wirelessNetmask = ""
        root.wirelessIp = ""
        root.wirelessMac = ""
        root.wiredEnabled = false
        root.wiredBroadcast = ""
        root.wiredNetmask = ""
        root.wiredIp = ""
        root.wiredMac = ""
        root.signalStrengthRatio = ""
    }

    function printStatus() {
        console.log("Wire Disconnected:", root.wireDisconnected)
        console.log("Wireless?", root.wirelessEnabled)
        console.log("Encrypted:",root.encrypted)
        console.log("IP Address:",root.wirelessIp)
        console.log("Broadcast:",root.wirelessBroadcast)
        console.log("Subnet Mask:",root.wirelessNetmask)
        console.log("Signal Strength:",root.signalStrength)
        console.log("MAC Address:",root.wirelessMac)
        console.log("Wired?",root.wiredEnabled)
        console.log("IP Address:",root.wiredIp)
        console.log("Broadcast:",root.wiredBroadcast)
        console.log("Subnet Mask:",root.wiredNetmask)
        console.log("MAC Address:",root.wiredMac)
    }

    function findInterfaceIndex(intf) {
        for (var i = 0; i < activeConnections.count; i++) {
            if (activeConnections.get(i).intf === intf) {
                return i
            }
        }
        return -1
    }

    function refreshList() {
        resetFields()
        if (wireWatcher.processState !== Process.Running) {
            console.log("starting wire watcher")
            wireWatcher.start()
        }

        debugLog("sending command... "+ifconfigProcess.program)
        refreshTimer.restart()

        fetchConfigured()

        if (root.wiredConfigured) {
            var wiredConn = {"intf": "p2p1", "essid": "Wired Ethernet", "encrypted": false}
            activeConnections.append(wiredConn)
            ifconfigProcess.start()
        }
        if (root.wirelessConfigured) {
            var conn = {"intf": "wlan0", "essid": "", "encrypted": false}
            activeConnections.insert(0, conn)
            ifconfigProcessWireless.start()
        }

        console.log("Active Conn:", activeConnections.count)

//        netpulseEnabledMode.value = options.netpulseEnabled ? true : false
//        browserEnabledMode.value = options.browserEnabled ? true : false
    }

    StorageMgr {
        id: storageMgr
    }

    Timer {
        id: refreshTimer
        interval: 1000
        repeat: false; running: false
    }

    Process{
        id: ifconfigProcess
        program: "ifconfig"
        arguments: ["p2p1"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "p2p1")
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }

    }

    Process {
        id: iwconfigProcess
        program: "iwconfig"
        arguments: ["wlan0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "iwconfig output: " + output)
            if(output.match(/wlan0/)) {
                var essidSearch = output.match(/ESSID:"(.+?)"\s/)
                root.currentESSID = essidSearch ? essidSearch[1] : qsTr("not connected","wireless network")
                var strengthMatch = output.match(/Link Quality=(.+?)\s/)
                if (strengthMatch) {
                    root.signalStrengthRatio = strengthMatch[1]
                }
            }
            wpaCliProcess.start()
        }
    }

    Process {
        id: ifconfigProcessWireless
        program: "ifconfig"
        arguments: ["wlan0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "wlan0")
            iwconfigProcess.start()
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }
    }

    Process {
        id: wpaCliProcess
        program: "sudo"
        arguments: ["wpa_cli", "status"]
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            if (output.indexOf("key_mgmt=NONE") >= 0) {
                root.encrypted = false
            } else {
                root.encrypted = true
            }
            if (root.wirelessEnabled) {
                var index = root.findInterfaceIndex("wlan0")
                if (index >= 0) {
                    activeConnections.setProperty(index, "essid", root.currentESSID)
                    activeConnections.setProperty(index, "encrypted", root.encrypted)
                } else {
                    debugLog("Warning: Wireless interface wlan0 not found in list")
                }
            }
        }
    }

    Process {
        id: wireWatcher
        program: "scripts/watch.sh"
        arguments: ["/sys/class/net/p2p1/carrier"]
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            output = output.trim()
            if (output === "1") {
                root.wireDisconnected = false
            } else if (output === "0") {
                root.wireDisconnected = true
            } else {
                console.log("wire watcher: unknown output:", output)
            }
        }
    }

    ListModel {
        id: activeConnections
        // interface is a reserved keyword in javascript
        //ListElement { intf: "wlan0"; encrypted: true; essid: "npwireless2.4" }
        //ListElement { intf: "p2p1"; encrypted: false; essid: "Wired Ethernet" }
    }

}
