import QtQuick 2.4
import Components 1.0

Item {
    id: root

    property real minValue: 0
    property real maxValue: 100
    property real value: 0
    property int precision: 0

    property bool clickableTrack: true
    property real stepValue: 1
    property int _minpos: 0
    property int _maxpos: (orientation == Qt.Horizontal) ? track.width : track.height - baseOffset
    property bool realtime: true  // emit values while slider is being moved
    property url thumbImage: "graphics/205/IPOD_Red_Dot.png"
    property url trackImage: ""
    property real trackImageOpacity: 1.0
    property url trackFillImage: ""
    property real trackFillImageOpacity: 1.0
    property bool showButtons: true
    property bool showValue: true
    property int valueOffsetX: 0
    property int valueOffsetY: 0
    property int buttonOffsetX: 0
    property int buttonOffsetY: 0
    property int buttonMargin: 20
    property int orientation: Qt.Horizontal
    property url decrementButtonImage: "graphics/Pace_Setter_Simple_Minus.png"
    property url incrementButtonImage: "graphics/Pace_Setter_Simple_Plus.png"
    property bool repeat: false
    property real thumbMouseAreaScale: 1.0
    property real buttonAreaScale: 2.0
    property int zeroOffset: 0 // make the slider "zero" value offset by some distance
    property int dragAreaMargin: 0
    property real baseOffset: 0.0
    property int baseIndex: 0 // this is the index that will be baseOffset units from the bottom
    property real knobOffset: calculateKnobOffset()

    function calculateKnobOffset() {
        if (!knobMouseArea.drag.active) {
            return sliderKnob.anchors.bottomMargin - 12
        } else {
            return track.height - sliderKnob.y - 36
        }
    }

    signal released()

    width: trackImage != "" ? trackGraphic.implicitWidth : (orientation == Qt.Horizontal ? 100 : 5)
    height: trackImage != "" ? trackGraphic.implicitHeight : (orientation == Qt.Horizontal ? 5 : 100)

    Component.onCompleted: {
        value = validateRange(value)
    }

    function setValue(newValue) {
        if(!precision) {
            var change = newValue - value
            change = (change/stepValue).toFixed(0)*stepValue
            newValue = value + change
        }
        else
            newValue = parseFloat(newValue).toFixed(precision)
        newValue = validateRange(newValue)
        value = newValue //newValue.toFixed(precision)
    }

    function validateRange(newValue) {
        if( newValue < root.minValue )
            newValue = root.minValue
        if( newValue > root.maxValue )
            newValue = root.maxValue
        return newValue
    }

    function positionFromValue(value) {
        var pos
        if (value < baseIndex)
            pos = (value - minValue)/(baseIndex - minValue) * baseOffset
        else
            pos = baseOffset + (value - baseIndex)/(maxValue - minValue - baseIndex) * ((_maxpos - _minpos) - baseOffset)
        return pos
    }

    function valueFromPosition(pos) {
        var value
        if (pos < baseOffset)
            value = minValue + Math.round((pos/baseOffset) * (baseIndex - minValue))
        else
            value = baseIndex + Math.round((pos - baseOffset) / ((_maxpos - _minpos) - baseOffset) * (maxValue - minValue - baseIndex))
        value = validateRange(value)
        return value
    }

    function decrementValue() {
        value = validateRange( value - stepValue )
    }

    function incrementValue() {
        value = validateRange( value + stepValue )
    }

    Item {
        id: track
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        width: root.width - (showButtons ? (10 + incrementButton.width + decrementButton.width + thumbImage.width/2) : 0)
        height: root.height - (showButtons ? (10 + incrementButton.height + decrementButton.height + thumbImage.height/2) : 0)
        z:-1 // place below increment/decrement buttons

        // track line when there's no background image
        Rectangle {
            id: track_line
            height: 3
            width: parent.width
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            color: "black"
            visible: root.trackImage != "" ? false : true
        }

        // image to display over the "filled" area of slider
        BorderImage {
            anchors.left: parent.left
            anchors.leftMargin: 1
            anchors.right: sliderKnob.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            source: root.trackFillImage
            opacity: trackFillImageOpacity
        }
        BorderImage {
            id: trackGraphic
            source: root.trackImage
            opacity: trackImageOpacity
            anchors {
                top: parent.top
                bottom: parent.bottom
                horizontalCenter: root.orientation == Qt.Vertical ? parent.horizontalCenter : undefined
            }
        }

        Text {
            id: valueDisplay
            visible: root.showValue
            //anchors { left: root.left ; leftMargin: root.valueOffsetX } //; bottom: sliderKnob.top ; bottomMargin: -valueOffsetY }
            text: root.value
        }

        // mouse area for clicking the track to move the slider
        MouseArea {
            id: track_mouse

            enabled: clickableTrack
            anchors.fill: parent
            anchors.topMargin: 20
            onPressed: {
                var position = (root.orientation == Qt.Horizontal ? (_minpos + mouseX ) : (_maxpos -_minpos - mouseY + 24))
                value = valueFromPosition( position )
                buzzer.buttonPress()
            }
            Rectangle{
                opacity: 0.3
                color: "green"
                anchors.fill: parent
                visible: main.showStimulusArea
            }
        }

        Image {
            id: sliderKnob

            anchors.left: parent.left
            anchors.leftMargin: positionFromValue(value) - width/2
            anchors.bottomMargin: positionFromValue(value) + 12
            source: root.thumbImage

            MouseArea {
                id: knobMouseArea
                property int knobMargin: (root.orientation == Qt.Horizontal ? (sliderKnob.width/2) : (sliderKnob.height/2))

                anchors.fill: parent
                anchors { topMargin: root.dragAreaMargin; bottomMargin: root.dragAreaMargin }
                scale: root.thumbMouseAreaScale

                drag {
                    target: sliderKnob ;
                    axis: (root.orientation === Qt.Horizontal) ? Drag.XAxis : Drag.YAxis;
                    // x boundaries
                    minimumX: _minpos - knobMargin
                    maximumX: _maxpos - knobMargin + 5
                    // y boundaries
                    minimumY: _minpos - knobMargin
//                    maximumY: _maxpos - knobMargin + 5    // +5 fudge factor need for some reason to allow full sliding to max value
                    maximumY: baseIndex > minValue ? track.height - knobMargin*2 : _maxpos - knobMargin*2 + 5
                }

                onPressed: {
                    sliderKnob.anchors.left = undefined
                    sliderKnob.anchors.bottom = undefined
                    buzzer.buttonPress()
                } // un-anchor to allow drag
                onReleased: {
                    if(root.orientation == Qt.Horizontal)
                        sliderKnob.anchors.left = sliderKnob.parent.left
                    else
                        sliderKnob.anchors.bottom = sliderKnob.parent.bottom
                    root.released()
                } // re-anchor at end of drag

                onMouseXChanged: {
                    if (!drag.active)
                        return
                    if( root.orientation == Qt.Horizontal )
                        setValue(valueFromPosition(sliderKnob.x + sliderKnob.width/2))
                }
                onMouseYChanged: {
                    if (!drag.active)
                        return
                    if( root.orientation != Qt.Horizontal )
                        setValue(valueFromPosition(_maxpos - _minpos - sliderKnob.y - sliderKnob.height/2 + 24))
                }

                Rectangle{
                    opacity: 0.3
                    color: "blue"
                    anchors.fill: parent
                    visible: main.showStimulusArea
                }
            }
        }
    }

    ImageButton {
        id: decrementButton

        visible: showButtons
        repeat: root.repeat
        anchors { left: parent.left ; leftMargin: -root.buttonMargin ; verticalCenter: parent.verticalCenter ; verticalCenterOffset: root.buttonOffsetY ; horizontalCenterOffset: root.buttonOffsetX ; topMargin: root.buttonMargin; bottomMargin: root.buttonMargin}
        downimage: root.decrementButtonImage
        upimage: root.decrementButtonImage
        onClicked: {
            root.decrementValue()
        }
        mouseAreaScale: root.buttonAreaScale
        z:0
    }

    ImageButton {
        id: incrementButton
        visible: showButtons
        repeat: root.repeat
        anchors { right: parent.right ; rightMargin: -root.buttonMargin ; verticalCenter: parent.verticalCenter ; verticalCenterOffset: root.buttonOffsetY ; horizontalCenterOffset: root.buttonOffsetX ; topMargin: root.buttonMargin; bottomMargin: root.buttonMargin }
        downimage: root.incrementButtonImage
        upimage: root.incrementButtonImage
        onClicked: {
            root.incrementValue()
        }
        mouseAreaScale: root.buttonAreaScale
        z:0
    }

    states: [
        State {
            name: "Horizontal"
            when: orientation == Qt.Horizontal

            PropertyChanges {
                target: track
                width: root.width - (showButtons ? (10 + incrementButton.width + decrementButton.width) : 0)
                height: root.height
            }

            AnchorChanges {
                target: track
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
            }

            PropertyChanges {
                target: track_line
                height: 3
                width: parent.width
            }

            AnchorChanges {
                target: sliderKnob
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left }
            }

            AnchorChanges {
                target: incrementButton
                anchors { right: parent.right ; verticalCenter: parent.verticalCenter }
            }

            AnchorChanges {
                target: decrementButton
                anchors { left: parent.left ; verticalCenter: parent.verticalCenter }
            }
        },
        State {
            name: "Vertical"
            when: orientation == Qt.Vertical

            PropertyChanges {
                target: track
                height: root.height - (showButtons ? (10 + incrementButton.height + decrementButton.height) : 0)
                width: root.width
            }

            AnchorChanges {
                target: track
                anchors {
                    left: undefined
                    right: undefined
                    top: root.top
                    bottom: root.bottom
                    horizontalCenter: root.horizontalCenter
                    verticalCenter: undefined
                }
            }

            PropertyChanges {
                target: track_line
                height: track.height;
                width: 3
                opacity: 0.5
            }

            AnchorChanges {
                target: sliderKnob
                anchors { verticalCenter: undefined ; left: undefined ; bottom: parent.bottom ; horizontalCenter: parent.horizontalCenter }
            }

            AnchorChanges {
                target: incrementButton
                anchors { bottom: track.top; left: undefined; right: undefined; horizontalCenter: parent.horizontalCenter ; verticalCenter: undefined}
            }

            AnchorChanges {
                target: decrementButton
                anchors { top: track.bottom ; left: undefined ; right: undefined ; horizontalCenter: parent.horizontalCenter ; verticalCenter: undefined}
            }

        }
    ]

}
