import QtQuick 2.4
import Process 0.1

// digital TV scanner
Item {
    id: root

    property string scanType: "digital"
    property bool scanning: false
    property int scanFreq: 0
    property int signalsFound: 0
    property real progress: 0
    property alias channels: scanServices
    property string tunerType
    property bool debug: false
    property bool aborted: false
    property bool useMock: false

    signal scanComplete()
    signal channelFound(variant chandata)

    function startScan(countryCode, isCable) {
        root.aborted = false
        root.scanFreq = 0
        root.signalsFound = 0
        root.progress = 0

        scanServices.clear()

        var args = buildArgs(countryCode, isCable)
        //var args = ["/data/scan_stderr2_AmFam.out","/data/scan_stdout_AmFam.out"]
        debugPrint("w_scan args:" + args)
        dvbscan.arguments = args
        root.scanning = true
        if (root.useMock)
            dvbscan.program = "bash"
        console.log("Starting", dvbscan.program, "with", dvbscan.arguments)
        dvbscan.start()
    }

    function stopScan() {
        root.aborted = true
        dvbscan.terminate()
        root.scanning = false
    }

    function buildArgs(countryCode, isCable) {
        if (root.useMock) {
            if (isCable)
                return ["scripts/scan_mock/scan_mock.sh", "atsc_us_digital_antenna.outp"]
            else
                return ["scripts/scan_mock/scan_mock.sh", "atsc_us_digital_antenna.outp"]
        }
        var args = ["-c", countryCode, "-k"]
        args= args.concat(["-t","3"]) //Use slowest tuning timeout to help with channel location
        if(tunerType == "ATSC" || tunerType == "QAM") {
            args = args.concat(["-f", "a"])
            if( isCable )
                args = args.concat(["-A", "2"]) // maybe want to scan both? (3)
        }
        else { // DVB, ISDB, or other
            if(isCable)
                args = args.concat(["-f", "c"])
            else
                args = args.concat(["-f", "t"])
            // currently no sat support
        }

        // Set delsys for ISDB/DTMB
        if (tunerType === "ISDB" || tunerType === "DTMB") {
            args = args.concat(["-d"])
        }

        // Filter out radio and encrypted channels
        args = args.concat(["-R", "0"])
        if (tunerType != "ISDB-T") {
            args = args.concat(["-E", "0"])
        }

        return args
    }

    function debugPrint(string) {
        if(root.debug)
            print(string)
    }

    ListModel {
        id: scanServices
    }

    Process {
        id: dvbscan

        program:  "w_scan"
        //program: "/usr/local/bin/truetouch/scripts/dvbscansim.sh"

        onFinished: {
            debugPrint("--------- digital tv scan complete.")
            root.scanComplete()
            root.scanning = false

            for(var i=0 ; i < scanServices.count ; i++) {
                var item = scanServices.get(i)
                debugPrint("name:" + scanServices.get(i).name + " program:" +scanServices.get(i).program + " freq:"+ scanServices.get(i).freq)
            }
        }

        function parseMplayerScanLine(line) {
            // colon separated fields
            var fields = line.split(":")
            var channame = fields[0]
            for( var j=0 ; j< scanServices.count ; j++) {
                // find the station name in the services listModel
                if(scanServices.get(j).name == channame) {
                    // append the channel to the tv list
                    debugPrint("found chan:"+scanServices.get(j).number + " " + scanServices.get(j).name)
                    var chanData = {name:fields[0], freq:fields[1], modulation:fields[2],program:fields[5] }
                    scanServices.set(j, chanData)
                }
            }
        }

        function parseKaffineScanLine(line) {
            var f = line.split("|")
            var constell = f[13]  // e.g. the 8 in 8VSB
            var frontend = f[7] // ATSC, Cable, Terrestrial
            var modulation
            if(frontend == "Atsc"){
                if(constell == "108")
                    modulation = "8VSB"
                else if(constell == "116")
                    modulation = "16VSB"
                else
                    modulation = constell + "VSB"
            }
            else if (frontend == "Cable") { // DVB-C, QAM
                modulation = constell+"QAM"
            }
            else if (frontend == "Terrestrial") { // DVB-T
                if(constell== "8")
                    modulation = "8QPSK"
                else if(constell == "-1")
                    modulation = "QAM"
                else
                    modulation = constell + "QAM"
            }

            console.log("array", JSON.stringify(f))
            var chanData = {enabled:"true", tunertype: root.tunerType, name:f[1], vpid:f[2], apids:f[3],
                ttypid:f[4], program:f[5], frontend: frontend, streamid:f[6], freq:""+(parseInt(f[8])*1000),
                symbolrate:f[9], fec: f[11] + "/" + f[14], inversion:f[12], modulation: modulation,
                bandwidth: f[15], transmode: f[16], guard: f[17], hier: f[18], number: f[19] }

            return chanData
        }

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            debugPrint( "scan output: " + output)
            output = output.trim()
            var lines = output.split("\n")
            for(var i=0 ; i < lines.length ; i++) {
                var line = lines[i]
                if(line[0] == "#")
                    continue // skip commented lines

                var chanData = parseKaffineScanLine(line)
                scanServices.append(chanData)
                root.channelFound(scanServices.get(scanServices.count - 1))
                debugPrint("appending channel: " + chanData.name + " freq:" + chanData.freq + " program:" + chanData.program + " modu:"+chanData.modulation +" number: " + chanData.number)
            }
        } // end onReadyReadStandardOutput

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            var lines = output.split("\n")
            for(var i=0 ; i < lines.length ; i++) {
                var freq
                var line = lines[i]
                debugPrint( "scan err output: " + line)
                if(line.match("signal ok"))
                    root.signalsFound++
                if(line.match(/\d+:/)) {
                    freq = String(line.split(":")[0])
                    if(freq > 0) {
                        debugPrint("freq from output:" + freq)
                        root.scanFreq = parseInt(freq)
                        root.progress = root.scanFreq / 900000
                    }
                }
                if(line.match("tune to")){
                    // keep track of current frequency to associate with names and numbers
                    if(root.tunerType == "ATSC" || root.tunerType == "QAM") {
                        // line format is tune to: 8VSB     f=xxxxxx kHz
                        freq = line.split("f=")[1]
                    }
                    else {
                        // line format is tune to: QAM_XXX     f =   xxxxxx kHz  IXXXBXCXXDXTXGXYX
                        freq = line.split("f = ")[1]
                        freq = freq.trim().split(" ")[0]
                    }
                    root.scanFreq = parseInt(freq.split(" ")[0])
                    root.progress = root.scanFreq / 900000
                }
            }
        } // end onReadyReadStandardError:
    } // end Process
}
