import QtQuick 2.4
import FileIO 1.0
import Process 0.1
import Components 1.0

Item {
    id: root

    property bool recording: false
    property bool copying: false
    property string recordFilename: ""
    property string captureDir: "/tmp/capture"

    function getRecordFilename() {
        var chan = tv.getChannelObject()
        var freq = chan.freq
        var filename = "f" + freq + ".ts"
        return filename
    }

    function startRecording() {
        if (!storageMgr.usbDeviceLoaded) {
            needUsbDevicePopup.showWithTimeout(3)
            return
        }
        if (!tv.playing) {
            return
        }
        tv.on = false

        root.copying = false
        root.recording = true
        root.recordFilename = root.getRecordFilename()
        console.log("Starting TV recording...")
        console.log("Saving to", root.captureDir + "/" + recordFilename)
        tsCaptureTimer.start()
    }

    Timer {
        id:tsCaptureTimer
        running: false
        repeat: false
        interval: 1000
        onTriggered: {
            tsCaptureProcess.prepareArgs()
            tsCaptureProcess.start()
        }
    }

    function stopRecording() {
        console.log("!!! Function is deprecated")
    }

    Process {
        id: tsCaptureProcess
        function prepareArgs() {
            // Elaborate process to convert stored channel info into format needed by tscap

            var args = ["-m"]
            if (tv.tunerType === "DVB-C")
                args = args.concat(["QAM"])
            else if (tv.tunerType === "DVB" || tv.tunerType === "ISDB")
                args = args.concat(["OFDM"])
            else
                args = args.concat(["ATSC"])

            var chan = tv.getChannelObject()
            if (tv.tunerType === "DVB" || tv.tunerType === "DVB-C" || tv.tunerType === "ISDB") {

                // FEC, modulation, inversion are used by both channel types
                if (chan.frontend === "Cable" || chan.frontend === "Terrestrial") {
                    // Inversion
                    var inv = parseInt(chan.inversion)
                    if (inv === 0)
                        args = args.concat(["INVERSION_OFF"])
                    else if (inv === 1)
                        args = args.concat(["INVERSION_ON"])
                    else
                        args = args.concat(["INVERSION_AUTO"])

                    // FEC
                    var fec = chan.fec.split("/")
                    if (fec.length > 0 && fec[0] !== "-1"){
                        // 34/0 => FEC_3_4
                        args = args.concat(["FEC_"+fec[0][0]+"_"+fec[0][1]])
                    } else {
                        args = args.concat(["FEC_AUTO"])
                    }
                    // Modulation
                    if (chan.modulation) {
                        if (chan.modulation.indexOf("QPSK") >= 0) {
                            args = args.concat("QPSK")
                        } else if (chan.modulation.indexOf("QAM") >= 0) {
                            // 64QAM => QAM_64
                            var num = chan.modulation.split("QAM")[0]
                            if (num)
                              args = args.concat(["QAM_"+num])
                            else
                              args = args.concat(["QAM_AUTO"])
                        } else {
                            args = args.concat(["QAM_AUTO"])
                        }
                    }
                }

                if (chan.frontend === "Cable") {
                    // Symbol Rate
                    args = args.concat([chan.symbolrate])

                } else { // Terrestrial
                    // Bandwidth
                    if (parseInt(chan.bandwidth) > 0) {
                        args = args.concat(["BANDWIDTH_"+chan.bandwidth+"_MHZ"])
                    }
                    // Transmission Mode
                    if (parseInt(chan.transmode) > 0) {
                        args = args.concat(["TRANSMISSION_MODE_"+chan.transmode+"K"])
                    } else {
                        args = args.concat(["TRANSMISSION_MODE_AUTO"])
                    }
                    // Guard Interval
                    if (parseInt(chan.guard) > 0) {
                        args = args.concat(["GUARD_INTERVAL_1_"+chan.guard])
                    } else {
                        args = args.concat(["GUARD_INTERVAL_AUTO"])
                    }
                    // Hierarchy
                    if (parseInt(chan.hier) > 0) {
                        args = args.concat(["HIERARCHY_"+chan.hier])
                    } else {
                        args = args.concat(["HIERARCHY_AUTO"])
                    }

                }
            } else if (tv.tunerType === "ATSC") {
                // Modulation
                if (chan.modulation.indexOf("VSB") >= 0) {
                    // 8VSB => 8VSB, 16VSB => 16VSB
                    args = args.concat([chan.modulation])
                } else if (chan.modulation.indexOf("QAM") >= 0) {
                    // 64QAM => QAM_64
                    var num = chan.modulation.split("QAM")[0]
                    if (num)
                        args = args.concat(["QAM_"+num])
                    else
                        args = args.concat(["QAM_AUTO"])
                } else {
                    args = args.concat(["8VSB"])
                }
            }

            args = args.concat(["-t", "30"])
            args = args.concat(["-q", chan.freq])
            args = args.concat(["-o", root.captureDir + "/" + root.recordFilename])

            console.log("Calling capture_ts with arguments", args)
            tsCaptureProcess.arguments = args
        }
        program: "scripts/capture_ts.sh"
        onReadyReadStandardOutput: {
            var data = String(tsCaptureProcess.readAllStandardOutput())
            data = data.trim()
            console.log("capture_ts stdout:", data)
            if (data.indexOf("COPY_COMPLETE") >= 0) {
                root.copying = false
            } else if (data.indexOf("CAPTURE_COMPLETE") >= 0) {
                root.recording = false
                root.copying = true
            }
        }
        onReadyReadStandardError: {
            var data = String(tsCaptureProcess.readAllStandardError())
            data = data.trim()
            console.log("capture_ts stderr:", data)
            if (data.indexOf("ERROR_NO_SPACE") >= 0) {
                noSpacePopup.showWithTimeout(3)
            }
        }
        onFinished: {
            console.log("capture_ts finished")
            root.copying = false
            root.recording = false
        }
    }

    MessagePopup {
        id: noSpacePopup
        message: qsTr("Could not record: Not enough space left on device") + application.emptyString
    }

    MessagePopup {
        id: needUsbDevicePopup
        message: qsTr("Insert USB device to record TV") + application.emptyString
    }
}
