import QtQuick 2.4
import ".."
import FreezeFrame 1.0

// Composite Entertainment view, showing TV, iPod and Netpulse views
Rectangle {
    id: root

    property string currentPage: "mainmenu" // Current netpulse view, default to menu

    // unregister all netpulse proxies when this view is destroyed
    function viewdestruct() {
        hidden()
        npMenu.viewdestruct()
        npTvMenu.viewdestruct()
        ondemand.viewdestruct()
        music.viewdestruct()
        tvView.viewdestruct()
    }

    // shown() is called by viewContainer when this view becomes the current view
    function shown() {
        // re-show the current netpulse page, if that's whats on.
        // Otherwise, a TV or iPod is probably visible.
        if(root.currentPage){
            netpulse.showPage(root.currentPage)
        }
        else if (tvView.visible)
            tvView.shown()
        else if (ipodView.visible)
            ipodView.shown()
    }

    function hidden() {
        // re-show the current netpulse page, if that's whats on.
        // Otherwise, a TV or iPod is probably visible.
        if(root.currentPage) {
            netpulse.hide()
        } else
        if (tvView.visible)
            tvView.hidden()
        else if (ipodView.visible)
            ipodView.hidden()

    }

    width:workoutViewContainer.contentAreaWidth-1
    height: workoutViewContainer.contentAreaHeight-1
    anchors.centerIn: parent
    objectName: "NP_MENU_VIEW_CONTAINER"

    Connections {
        target: netpulse
        onVisibleChanging: {
            if(!willBeVisible && (netpulse.currentPage === root.currentPage) ) {
                freezeframe.capture()
            }
        }
    }

    FreezeFrame {
        id: freezeframe

        anchors.fill: parent
        frozen: (!npMenu.visible && !npTvMenu.visible && !ondemand.visible && !music.visible) || netpulse.frozen
        visible: workoutViewContainer.isMoving || netpulse.frozen  // avoid excessive repaints by hiding when not needed
    }

    NetpulseEntertainmentMenu {
        id: npMenu
        // follow view changes received by the Entertainment menu proxy,
        // so we can switch back to the most recent when this view is shown
        onViewChanged: {
            switch(page) {
                case "ipod": {
                    ipodView.visible = true;
                    tvView.visible = false
                    ipodView.shown()
                    root.currentPage = "";
                    break;
                }
                case "games": {
                    // go back to mainmenu when games is selected (not yet implemented)
                    root.currentPage = "mainmenu";
                    netpulse.showPage("mainmenu");
                    break;
                }
                case "ondemand": {
                    root.currentPage = page;
                    break;
                }
                case "music": {
                    root.currentPage = page;
                    break;
                }
                case "watch": {
                    root.currentPage = page;
                    break;
                }
                case "tv": {
                    root.currentPage = page;
                    tvView.visible = true;
                    break;
                }

                default: {
                    root.currentPage = page
                    break;
                }
            } // end switch(page)
        } // end onViewChanged
    }

    NetpulseTv {
        id: npTvMenu
        onChannelSelected: {
            tv.tuneChannelID(id);
            tvView.channelID = id
            root.currentPage = ""
            tvView.visible = true
        }
    }

    TvViewContainer {
        id: tvView
        anchors.fill: parent
        visible: false

        onVisibleChanged: {
            if(visible)
                tvView.shown()
            else
                tvView.hidden()
        }

        onStopped: { // return to Entertainment menu when TV stopped
            if(visible) {
                visible = false
                root.currentPage = "mainmenu"
                netpulse.showPage("mainmenu")
            }
        }
    }

    IPodViewContainer {
        id: ipodView

        showCloseButton: true

        onVisibleChanged: {
            if(visible)
                ipodView.shown()
            else
                ipodView.hidden()
        }

        onClosed: { // return to netpulse entertainment menu when ipod closed
            visible = false
            root.currentPage = "mainmenu"
            netpulse.showPage("mainmenu")
        }

        anchors.fill: parent
        visible: false
    }

    // The following proxy elements allow switching directly to these NP views
    // when swiping away, then back to the Entertainment view.
    NetpulseWindow {
        id: ondemand
        absx: workoutViewContainer.x+23
        absy: workoutViewContainer.y+23
        anchors.fill: parent

        page: "ondemand"
    }

    NetpulseWindow {
        id: music
        page: "music"
        anchors.fill: parent
        absx: workoutViewContainer.x+23
        absy: workoutViewContainer.y+23
    }
}
